@@ -1,9 +1,9 @@
 ##======================================================================
-## Author: Yves Deville
+## AUTHOR: Yves Deville <deville.yves@alpestat.com>
 ##
 ## Find ML estimate of a two parameter Lomax distribution using a
-## sample 'x'. The likelihood is concentrated with respect to the shape
-## parameter 'alpha' and maximised with respect to 'beta'.
+## sample 'x'. The likelihood is concentrated with respect to the
+## shape parameter 'alpha' and maximised with respect to 'beta'.
 ##
 ## When 'scaleData' is TRUE, the sample is divided by its mean in order
 ## to make the optimisation safer.
@@ -16,33 +16,28 @@
 ##=======================================================================
 
 `flomax` <- function(x,
-                     info.observed = FALSE,
+                     info.observed = TRUE,
                      plot = FALSE,
                      scaleData = TRUE,
                      cov = TRUE) {
 
     Cvg <- TRUE
-    if (any(x <= 0)) stop("all elements in 'x' must be > 0")  
+    if (any(x <= 0.0)) stop("all elements in 'x' must be > 0")  
     
     parnames <- c("shape", "scale")
     n <- length(x)
     M1 <- mean(x)
     
     if (scaleData) {
-        ## xUnscaled <- x    ## useful ???
         x <- x / M1
-        CV <- sqrt(1 - 1/n) * sd(x)
+        CV <- sqrt(1.0 - 1.0 / n) * sd(x)
         cLogLik <- - n * log(M1)
-        trans <- diag(c(1, 1 / M1))
+        trans <- diag(c(1.0, 1.0 / M1))
     } else {
-        CV <- sqrt(1 - 1/n) * sd(x) / M1
+        CV <- sqrt(1.0 - 1.0 / n) * sd(x) / M1
     }
     
     if (CV < 1.00) stop("CV < 1. Estimation impossible for \"lomax\"")
-    ## if (CV < 1.001) {
-    ##     warning(sprintf("small CV value: %7.4f.", CV),
-    ##             " Estimation may not converge")
-    ## }
     
     ## compute an upper bound for beta. M1 <- mean(x) was done before
     M2 <- mean(x^2)
@@ -52,12 +47,12 @@
         ## 1st condition ensures ell(beta) > ell(infty) the second
         ## is better
         ## betaRoots <- polyroot(c(2*M3, 2*M3 - 3*M2, -3*(M2 - 2)))
-        betaRoots <- polyroot(c(M3, M3 - M2, 1 - M2/2))
+        betaRoots <- polyroot(c(M3, M3 - M2, 1.0 - M2 / 2))
     } else {
         ## 1st condition ensures ell(beta) > ell(infty), the second
         ## is better
         ## betaRoots <- polyroot(c(2*M1*M3, 2*M3 - 3*M2*M1, -3*(M2 - 2*M1^2))) 
-        betaRoots <- polyroot(c(M1 * M3, M3 -M1 * M2, M1^2 - M2/2))
+        betaRoots <- polyroot(c(M1 * M3, M3 - M1 * M2, M1^2 - M2 / 2))
     }
     
     betaLower <- 0.15 * min(x)
@@ -84,29 +79,33 @@
     if (plot) cov <- TRUE
     
     if (cov) {
+        
         log2L <- function (alpha, beta) {
             xmod <- x / beta
-            xmod1 <- 1 + xmod
+            xmod1 <- 1.0 + xmod
             s1 <- sum(log(xmod1))
             s2 <- sum(xmod / xmod1)
             alpha1 <- 1.0 + alpha
             
             logL <- n * log(alpha / beta)  - alpha1 * s1 
             
-            dlogL <- c(shape = n /alpha - s1,
-                       scale = ( -n + alpha1 * s2 ) / beta )
+            dlogL <- c(shape = n / alpha - s1,
+                       scale = (-n + alpha1 * s2) / beta)
             
-            d2logL <- array(0, dim = c(2, 2), dimnames = list(parnames, parnames))
-            d2logL["shape", "shape"] <- -n / alpha /alpha
+            d2logL <- array(0, dim = c(2L, 2L), dimnames = list(parnames, parnames))
+            d2logL["shape", "shape"] <- -n / alpha / alpha
             d2logL["shape", "scale"] <- s2 / beta
             d2logL["scale", "shape"] <- d2logL["shape", "scale"]
             d2logL["scale", "scale"] <-
-                (n  - alpha1*s2  -alpha1 * sum(xmod / xmod1 / xmod1)) / beta / beta
+                (n  - alpha1 * s2  -alpha1 * sum(xmod / xmod1 / xmod1)) / beta / beta
             
             if (scaleData) {
                 logL <- logL + cLogLik
                 dlogL <- trans %*% dlogL 
                 d2logL <- trans %*% d2logL %*% trans
+                ## added on 2015-12-17 Yves
+                rownames(dlogL) <- parnames
+                dimnames(d2logL) <- list(parnames, parnames)
             } 
             
             list(logL = logL,
@@ -116,17 +115,9 @@
 
     } 
     
-    ## checks <- unlist(sapply(interv, dlogLc))
-    ## if ( (checks[1] < 0) ||  (checks[2] > 0) ) {
-    ##     cat("flomax\n")
-    ##     print(checks)
-    ##     warning("no interval found to maximise loglik")
-    ##     Cvg <- FALSE
-    ## }
-    
     res <- optimize(f = logLc, interval = interv, maximum = TRUE)
     beta.hatS <- res$maximum
-    alpha.hat <- 1/ mean(log(1 + x / beta.hatS))
+    alpha.hat <- 1.0 / mean(log(1.0 + x / beta.hatS))
 
     if (scaleData) beta.hat <- M1 * beta.hatS
     else beta.hat <- beta.hatS
@@ -140,14 +131,15 @@
                     cvg = Cvg))
     }
 
-    ## Remind that log2L is called with scaled beta. The
+    ## Remind that 'log2L' is called with scaled 'beta'. The
     ## derivatives are corrected for the scaling.
     res2 <- log2L(alpha = alpha.hat, beta = beta.hatS)
     
     if (info.observed) {
         info <- -res2$d2logL
         vcov <- try(solve(info))
-        if (!inherits(vcov, "try-error")) {
+        ## changed 2015-12-17: the test was wrong!!!
+        if (inherits(vcov, "try-error")) {
             vcov <- NULL
             sds <- NULL
             warning("hessian could not be inverted")
@@ -155,16 +147,16 @@
             sds <- sqrt(diag(vcov))
         }
     } else {
-        a1 <- alpha.hat + 1
-        a2 <- alpha.hat + 2
-        i11 <- 1 / alpha.hat / alpha.hat
-        i12 <- -1 / beta.hat / a1
+        a1 <- alpha.hat + 1.0
+        a2 <- alpha.hat + 2.0
+        i11 <- 1.0 / alpha.hat / alpha.hat
+        i12 <- -1.0 / beta.hat / a1
         i22 <- alpha.hat / a2 / beta.hat / beta.hat
         info <- matrix(n * c(i11, i12, i12, i22), nrow = 2L, ncol = 2L)
         colnames(info) <- rownames(info) <- parnames
         c11 <- alpha.hat^2 * a1^2
         c12 <- alpha.hat * a1 * a2 * beta.hat
-        c22 <- a1^2 * a2  * beta.hat^2 / alpha.hat
+        c22 <- a1^2 * a2 * beta.hat^2 / alpha.hat
         vcov <- matrix(c(c11, c12, c12, c22) / n, nrow = 2L, ncol = 2L) 
         colnames(vcov) <- rownames(vcov) <- parnames
         sds <- sqrt(diag(vcov))
@@ -175,8 +167,8 @@
         dlogLc <- function (beta) {
             xmod <- x / beta
             R <- mean(log(1.0 + xmod))
-            dR <- -mean( xmod / (1.0 + xmod) ) / beta
-            -n * ((R + 1.0) * dR/R + 1.0 / beta) 
+            dR <- -mean(xmod / (1.0 + xmod)) / beta
+            -n * ((R + 1.0) * dR / R + 1.0 / beta) 
         }
         
         if (scaleData) beta.sol <- beta.hat / M1
@@ -189,7 +181,7 @@
         dfs <- sapply(betas, dlogLc)
         
         ind <- 1L:length(betas)
-        ind <- dfs < 20
+        ind <- (dfs < 20)
         Stext <- ifelse(scaleData, "(scaled data)", "")
         
         ## now plot logLik derivative and logLik
@@ -242,7 +234,7 @@
 }
 
 ##=======================================================================
-## Author: Yves Deville
+## AUTHOR: Yves Deville
 ##
 ## Find ML estimate of a two parameter Lomax distribution WITH KNOWN
 ## SHAPE.
@@ -255,98 +247,133 @@
 ##=======================================================================
 
 `flomax1` <- function(x,
-                      shape = 4,
-                      info.observed = FALSE,
+                      shape = 4.0,
+                      info.observed = TRUE,
+                      scaleData = TRUE,
+                      cov = TRUE,
                       plot = FALSE) {
 
-  Cvg <- TRUE
+    Cvg <- TRUE
   
-  if (any(x <= 0)) stop("all elements in 'x' must be >0")  
-  
-  n <- length(x)
-  CV <- sqrt(1 - 1/n) * sd(x) / mean(x)
-  
-  if (CV < 1.05) {
-    warning(sprintf("small CV value: %7.4f.", CV),
-            " Estimation may not converge!")
-  }
-
-  alpha <- shape
-  alpha1 <- shape + 1
-  
-  ## could be used to find a zero
-  dlogL1 <- function (beta) {
-    xmod <- x / beta
-    R <- mean(log(1.0 + xmod))
-    S1 <- mean( xmod / (1.0 + xmod) )
-    n*( - 1.0  + alpha1 * S1 )/beta 
-  }
-  
-  logL1 <- function (beta) {
-    xmod <- x/beta
-    R <- mean(log(1.0 + xmod))
-    n*( log(alpha) - log(beta) -alpha1* R )
-  }
-  
-  log2L1 <- function (beta) {
-    xmod <- x/beta
-    xmod1 <- 1 + xmod
-    R <- mean(log(xmod1))
-    S1 <- mean(xmod/xmod1)
-    alpha1 <- 1.0 + alpha
+    if (any(x <= 0)) stop("all elements in 'x' must be >0")  
     
-    logL <- n*( log(alpha) - log(beta)  - alpha1 * R)
-    dlogL <- n*( -1 + alpha1 * S1 )/beta 
-    d2logL <- n*( 1  - alpha1*S1  -alpha1*mean(xmod / xmod1 / xmod1) ) /beta/beta
+    n <- length(x)
+    M1 <- mean(x)
+
+    if (scaleData) {
+        x <- x / M1
+        CV <- sqrt(1.0 - 1.0 / n) * sd(x)
+        cLogLik <- - n * log(M1)
+        trans <- 1.0 / M1
+    } else {
+        CV <- sqrt(1.0 - 1.0 / n) * sd(x) / M1
+    }
+    N1 <- mean(1.0 / x)
     
-    list(logL = logL,
-         dlogL = dlogL,
-         d2logL = d2logL)
-  }
-
-  interv <- c(1e-8, 20*mean(x))
-  checks <- unlist(sapply(interv, dlogL1))
-  if ( (checks[1] < 0) ||  (checks[2] > 0) ) {
-    warning("no interval found to maximise loglik")
-    Cvg <- FALSE
-  }
-  res <- optimize(f = logL1, interval = interv, maximum = TRUE)
-  beta.hat <- res$maximum
-  loglik <- res$objective
-  alpha.hat <- 1/ mean(log(1 + x / beta.hat))
-
-  res2 <- log2L1(beta = beta.hat)
-              
-  if (info.observed) {
-    info <- -res2$d2logL
-  } else {
-    info <- n*alpha.hat / (alpha.hat + 2) / beta.hat / beta.hat
-  }
-  cov <- 1/info
-  sds <- sqrt(cov)
+    alpha <- unname(shape)
+    alpha1 <- shape + 1.0
     
-  if (plot) {
-    betas <- seq(from = interv[1], to = interv[2], length = 200)
-    fs <- sapply(betas, logL1)
-    dfs <- c(NA, diff(fs)/diff(betas))
-    ind <- 1L:length(betas)
-    ind <- dfs < 20
-    plot(betas[ind], dfs[ind],
-         type = "l", lty = "dotted",
-         main = "derivative of the log-likelihood",
-         xlab = "beta (scale)", ylab = "dlogL")
-    lines(betas[ind], sapply(betas[ind], dlogL1), col = "red")
-    abline(h = 0)
-    abline(v = beta.hat)
-  }
+    ## could be used to find a zero
+    dlogL1 <- function (beta) {
+        xmod <- x / beta
+        R <- mean(log(1.0 + xmod))
+        S1 <- mean(xmod / (1.0 + xmod))
+        n * (-1.0  + alpha1 * S1) / beta 
+    }
+    
+    logL1 <- function (beta) {
+        xmod <- x / beta
+        R <- mean(log(1.0 + xmod))
+        n * (log(alpha) - log(beta) - alpha1 * R)
+    }
 
-  list(estimate = c(scale = beta.hat),
-       sd = sds,
-       CV = CV,
-       loglik = loglik,
-       dloglik = res2$dlogL,
-       cov = cov,
-       info = info,
-       cvg = Cvg)
-  
+    if (plot) cov <- TRUE
+    
+    if (cov) {
+        log2L1 <- function (beta) {
+            xmod <- x / beta
+            xmod1 <- 1.0 + xmod
+            R <- mean(log(xmod1))
+            S1 <- mean(xmod / xmod1)
+            alpha1 <- 1.0 + alpha
+            
+            logL <- n * (log(alpha) - log(beta)  - alpha1 * R)
+            dlogL <- n * (-1.0 + alpha1 * S1) / beta 
+            d2logL <- n * (1.0  - alpha1 * S1  - alpha1 * mean(xmod / xmod1 / xmod1)) /
+                beta / beta
+            
+            if (scaleData) {
+                logL <- logL + cLogLik
+                dlogL <- trans * dlogL 
+                d2logL <- trans * d2logL * trans
+            }
+            
+            list(logL = logL,
+                 dlogL = dlogL,
+                 d2logL = d2logL)
+        }
+    }
+        
+    if (scaleData) interv <- c((1.0 - 1.0 / alpha1) / N1, alpha1)
+    else interv <- c((1.0 - 1.0 / alpha1) / N1, M1 * alpha1)
+    
+    checks <- unlist(sapply(interv, dlogL1))
+    if ((checks[1] < 0) || (checks[2] > 0)) {
+        warning("no interval found to maximise loglik")
+        Cvg <- FALSE
+    }
+    
+    res <- optimize(f = logL1, interval = interv, maximum = TRUE,
+                    tol = .Machine$double.eps^0.3)
+    beta.hatS <- res$maximum
+    alpha.hat <- alpha
+    
+    if (scaleData) beta.hat <- M1 * beta.hatS
+    else beta.hat <- beta.hatS
+
+    if (!cov) {
+        loglik <- res$objective
+        if (scaleData) loglik <- loglik + cLogLik
+        return(list(estimate = c(scale = beta.hat),
+                    CV = CV,
+                    loglik = loglik,
+                    cvg = Cvg))
+    }
+    
+    res2 <- log2L1(beta = beta.hatS)
+    loglik <- res2$logL
+        
+    if (info.observed) {
+        info <- -res2$d2logL
+    } else {
+        info <- n * alpha.hat / (alpha.hat + 2.0) / beta.hat / beta.hat
+    }
+    
+    cov <- 1.0 / info
+    sds <- sqrt(cov)
+    
+    if (plot) {
+        Stext <- ifelse(scaleData, "(scaled data)", "")
+        betas <- seq(from = interv[1], to = interv[2], length = 200)
+        fs <- sapply(betas, logL1)
+        dfs <- c(NA, diff(fs) / diff(betas))
+        ## ind <- 1L:length(betas)
+        ind <- (dfs < 20)
+        plot(betas[ind], dfs[ind],
+             type = "l", lty = "dotted", 
+             main = sprintf("'Lomax' log-lik derivative %s", Stext),
+             xlab = "beta (scale)", ylab = "dlogL")
+        lines(betas[ind], sapply(betas[ind], dlogL1), col = "orangered")
+        abline(h = 0, v = beta.hatS, col = "SpringGreen3")
+    }
+    
+    list(estimate = c(scale = beta.hat),
+         sd = sds,
+         CV = CV,
+         loglik = loglik,
+         dloglik = res2$dlogL,
+         cov = cov,
+         info = info,
+         cvg = Cvg)
+    
 }
